from time import gmtime,strftime,strptime
from qt import *
from qtxml import *

#
# These classes handle the reading and writing of property lists (plists)
# to/from XML
#
class PListReader:
	def parse(self,filename):
		dom = QDomDocument('plist')
		file = QFile(filename)
		if not file.open(IO_ReadOnly):
			# XXX DSM throw error!
			print 'failed open'
			return []
		if not dom.setContent(file):
			# XXX DSM throw error!
			print 'failed parse'
			file.close()
			return []
		file.close()
		return self.element(dom.documentElement())

	#
	# handle items of the form <tag>text</tag>, return the text
	#
	def text(self,element):
		return str(element.toElement().firstChild().nodeValue())
	
	#
	# handle an arbitrary element
	#
	def element(self,element):
		element = element.toElement()
		if element.isNull():
			return None
		tag = str(element.tagName())
		if tag=='plist' or tag=='array':
			array = []
			child = element.firstChild()
			while not child.isNull():
				array.append(self.element(child))
				child = child.nextSibling()
			return array
		elif tag=='dict':
			dict = {}
			key = element.firstChild()
			while not key.isNull():
				value = key.nextSibling()
				dict[str(self.text(key))] = self.element(value)
				key = value.nextSibling()
			return dict
		elif tag=='integer':
			return int(self.text(element))
		elif tag=='real':
			return float(self.text(element))
		elif tag=='date':
			return strptime(self.text(element),"%Y-%m-%dT%H:%M:%SZ")
		elif tag=='string' or tag=='data':
			return self.text(element)
		elif tag=='true':
			return True
		elif tag=='false':
			return False
		
class PListWriter:
	def unparse(self,plist,filename):
		doc = QDomDocument('plist')
		root = doc.createElement('plist')
		root.setAttribute('version','1.0')
		doc.appendChild(root)
		self.unparseObject(doc,root,plist[0])
		file = QFile(filename)
		if not file.open(IO_WriteOnly):
			# XXX DSM throw error
			return
		file.writeBlock(str(doc.toString()))
		file.close()
	
	#
	# create atomic elements of the form <tag>text</tag>
	#
	def atom(self,doc,parent,tag,text):
		obj = doc.createElement(tag)
		parent.appendChild(obj)
		obj.appendChild(doc.createTextNode(text))
		
	def unparseObject(self,doc,parent,obj):
		objClass = type(obj).__name__
		if objClass=='list':
			element = doc.createElement('array')
			parent.appendChild(element)
			for item in obj:
				self.unparseObject(doc,element,item)
		elif objClass=='dict':
			element = doc.createElement('dict')
			parent.appendChild(element)
			for key in obj.keys():
				self.atom(doc,element,'key',key)
				self.unparseObject(doc,element,obj[key])
		elif objClass=='int' or objClass=='long':
			self.atom(doc,parent,'integer',str(obj))
		elif objClass=='float':
			self.atom(doc,parent,'real',str(obj))
		elif objClass=='bool':
			if obj:
				parent.appendChild(doc.createElement('true'))
			else:
				parent.appendChild(doc.createElement('false'))
		elif objClass=='struct_time':
			self.atom(doc,parent,'date',strftime("%Y-%m-%dT%H:%M:%SZ",obj))
		elif objClass=='str':
			self.atom(doc,parent,'string',obj)
		else:
			parent.appendChild(doc.createElement(objClass))

if __name__ == '__main__':
	filename = "../Music/LSongs/LSongsMusicLibrary.xml"
	reader = PListReader()
	plist = reader.parse(filename)
	print plist
	writer = PListWriter()
	writer.unparse(plist,filename)
