from kdeemul import *
from utils import *
from backtick import *
import paths

class LExportBase(QDialog):
	def __init__(self, parent, keys):
		QDialog.__init__(self, parent, "Lphoto", True)
		self.lib = parent.lib
		self.settings = parent.settings
		self.count = len(keys)
		self.byteSize = self.lib.getTotalSize(keys);
		self.keys = keys

	def __isUnique(self, list, filename):
		for item in list:
			if item == filename:
				return False
		return True
		
	def fixName(self, list, filename):
		# makes sure name is unique
		(a,b) = os.path.splitext(filename)
		
		# add extension if not present
		if b == None or len(b) == 0:
			b = ".jpg"
		
		newName = a + b
		check = str(os.path.join(dir, newName))
		i = 1
		while (not self.__isUnique(list, check)):
			newName = a + str(i) + b
			check = os.path.join(dir, newName)
			i = i + 1
		return check
	
	def initSizeChoices(self):
		# init sizeCombo
		self.sizes = [
		[i18n("Small (320x240)"), QSize(320,240), 65000],
		[i18n("Medium (640x480)"), QSize(640,480), 124000],
		[i18n("Large (1200x900)"), QSize(1200,900), 180000],
		[i18n("Full (Actual Size)"), None, None]
		]
		
		for size in self.sizes:
			self.sizeCombo.insertItem(size[0])
		# select last item
		self.sizeCombo.setCurrentItem(self.sizeCombo.count()-1)

	def buildSpacer(self, grid, r, c, s):
		grid.addRowSpacing(r,s)
		return r
		
	def buildSizeInGrid(self, grid, r, c):
		bg = QButtonGroup("ImageSize", self)
		bg.hide()
		l = QLabel("Image Size:", self)
		grid.addWidget(l,r,c)

		hl = QHBoxLayout(self, 0, 0)
		grid.addLayout(hl,r,c+1)	
		self.sizeRadio = QRadioButton("", self)
		self.sizeRadio.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
		bg.insert(self.sizeRadio, 0)
		hl.addWidget(self.sizeRadio)
		hl.addSpacing(5)

		self.sizeCombo = QComboBox(self)
		hl.addWidget(self.sizeCombo)
		
		self.customRadio = QRadioButton("Custom Size", self)
		grid.addWidget(self.customRadio,r+1,c+1)
		bg.insert(self.customRadio, 1)
		
		gl = QGridLayout(self, 2, 2, 0)
		grid.addMultiCellLayout(gl, r+2, r+3, c+1, c+1)
		l = QLabel(i18n("Max Width: "), self)
		l.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
		gl.addWidget(l, 0, 0)
		
		self.widthEdit = QLineEdit(self)
		self.widthEdit.setValidator(QIntValidator(10, 12000, self))
		self.widthEdit.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
		gl.addWidget(self.widthEdit, 0, 1)

		l = QLabel(i18n("Max Height: "), self)
		l.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
		gl.addWidget(l, 1, 0)
		self.heightEdit = QLineEdit(self)
		self.heightEdit.setValidator(QIntValidator(10, 12000, self))
		self.heightEdit.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
		gl.addWidget(self.heightEdit, 1, 1)
		
		return r+3
		
	def buildPathInGrid(self, grid, r, c):
		l = QLabel(i18n("Export Folder: "), self)
		grid.addWidget(l,r,c)
		hl = QHBoxLayout()
		grid.addLayout(hl,r,c+1)
		self.exportEdit = QLineEdit(self)
		hl.addWidget(self.exportEdit)
		self.exportPathButton = QPushButton("Change Folder...", self)
		hl.addWidget(self.exportPathButton)
		return r
	
	def buildNameInGrid(self, grid, r, c):
		l = QLabel(i18n("Filenames:"), self)
		grid.addWidget(l,r,c)
		bg = QButtonGroup("Filenames", self)
		bg.hide()
		self.titleRadio = QRadioButton(i18n("Use Title as filename"), self)
		bg.insert(self.titleRadio);
		grid.addWidget(self.titleRadio, r, c+1)
		self.origRadio = QRadioButton(i18n("Use original filename"), self)
		bg.insert(self.origRadio);
		grid.addWidget(self.origRadio, r+1, c+1)
		return r+1

	def buildInfoLabelsInGrid(self, grid, r, c):
		if self.count == 1:
			t = i18n("1 photo")
		else:
			t = i18n("%d photos" % (self.count))
		l = QLabel(t, self)
		grid.addWidget(l, r, c+1)
		self.sizeLabel = QLabel("", self)
		grid.addWidget(self.sizeLabel, r+1, c+1)
		return r+1
		self.sizes[self.sizeCombo.currentItem()][1]

	def slotChangeSize(self, i):
		self.sizeRadio.setChecked(True)
		self.computeEstimatedSize()
		
	def slotRadioToggle(self, b):
		if b:
			self.computeEstimatedSize()
			
	def slotSizeEdit(self, s):
		self.customRadio.setChecked(True)
		self.computeEstimatedSize()

	def slotShowPathDialog(self):
		path = KFileDialog.getExistingDirectory(self.exportEdit.text(), None, i18n("Select your export directory"))
		if path:
			self.exportEdit.setText(path)
			

	def computeEstimatedSize(self):
		if (self.sizeRadio.isChecked()):
			self.size = self.sizes[self.sizeCombo.currentItem()][1]
		else:
			w = self.widthEdit.text()	# returns QString
			h = self.heightEdit.text()	# returns QString
			if w and h:
				self.size = QSize(int(str(w)), int(str(h)))
			else:
				self.size = None

		if self.size:  # if no size can be found, use filesize
			b = self.count * (0.75 * self.size.width() * self.size.height() + 2500)
		else:
			b = self.byteSize
		self.sizeLabel.setText(u"%s %s" % (i18n("Estimated Total Size: "), LUtils.humanByteOutput(b)))
		
	def serializeSizeSettings(self, category, bRestore):
		self.settings.serializeRadioGroup([self.sizeRadio, self.customRadio], category, "usesize", bRestore, default=0)
		self.settings.serializeEdit(self.widthEdit, category, "width", bRestore)
		self.settings.serializeEdit(self.heightEdit, category, "height", bRestore)
		self.settings.serializeCombo(self.sizeCombo, category, "sizeIndex", bRestore)
##~ 		
##~ 		
##~ 		if bRestore:
##~ 			if self.settings.readEntryBool(category, "usesize", True):
##~ 				self.sizeRadio.setChecked(True)
##~ 			else:
##~ 				self.customRadio.setChecked(True)
##~ 			self.sizeCombo.setCurrentItem(self.settings.readEntryInt(category, "sizeindex", self.sizeCombo.count()-1))
##~ 			i = self.settings.readEntryInt(category, "width", 0)
##~ 			if i > 0:
##~ 				self.widthEdit.setText(str(i))
##~ 			i = self.settings.readEntryInt(category, "height", 0)		# init dialog settings
##~ 			if i > 0:
##~ 				self.heightEdit.setText(str(i))
##~ 		else: # writing
##~ 			self.settings.writeEntryBool(category, "usesize", self.sizeRadio.isChecked())
##~ 			self.settings.writeEntryInt(category, "sizeindex", self.sizeCombo.currentItem())
##~ 			self.settings.writeEntryInt(category, "width", self.widthEdit.text())
##~ 			self.settings.writeEntryInt(category, "height", self.heightEdit.text())
	
##~ 	def serializeCombo(self, widget, category, key, bRestore):
##~ 		if bRestore:
##~ 			s = self.settings.readEntry(category, key, None)
##~ 			if s:
##~ 				i = widget.listBox().index(widget.listBox().findItem(s))
##~ 				widget.setCurrentItem(i)
##~ 		else:
##~ 			self.settings.writeEntry(category, key, str(widget.currentText()))
##~ 			
##~ 	def serializeEdit(self, widget, category, key, default=None):
##~ 		if default:
##~ 			widget.setText(self.settings.readEntry(category, key, str(default)))
##~ 		else:
##~ 			self.settings.writeEntry(category, key, str(widget.text()))
##~ ##~ 			return widget.text()
		
##~ 	def serializeFolderSettings(self, category, default):
##~ 		
##~ 		self.serializeEdit(self.exportEdit, category, "folder", default)

##~ 	def serializeNameSettings(self, category, bRestore):
##~ 		self.settings.serializeRadioGroup([self.titleRadio, self.origRadio], category, "usetitle", bRestore)
##~ ##~ 		if bRestore:
##~ ##~ 			if self.settings.readEntryBool(category, "usetitle", True):
##~ ##~ 				self.titleRadio.setChecked(True)
##~ ##~ 			else:
##~ ##~ 				self.origRadio.setChecked(True)
##~ ##~ 		else:
##~ ##~ 			self.settings.writeEntryBool("AlbumExport", "usetitle", self.useTitle)

class LAlbumExport(LExportBase):
	def __init__(self, parent, lib, albumPath):
		self.albumPath = albumPath
		keys = lib.getAlbum(self.albumPath)['KeyList']
		LExportBase.__init__(self, parent, keys)
		self.setCaption(i18n("Export Album"))
		
		gl = QGridLayout(self,14,2,5)

		r = self.buildPathInGrid(gl, 0, 0)
		r = self.buildSpacer(gl, r+1, 0, 10)
		r = self.buildNameInGrid(gl, r+1, 0)
		r = self.buildSpacer(gl, r+1, 0, 10)
		r = self.buildSizeInGrid(gl, r+1, 0)
		r = self.buildSpacer(gl, r+1, 0, 10)
		r = self.buildInfoLabelsInGrid(gl, r+1, 0)	

		r = self.buildSpacer(gl, r+1, 0, 10)
		
		hl = QHBoxLayout()
		gl.addLayout(hl,r+1,1)
		hl.addStretch()
		self.exportButton = QPushButton(i18n("Export"), self)
		hl.addWidget(self.exportButton)
		self.exportButton.setDefault(True)
		hl.addSpacing(5)
		self.cancelButton = QPushButton(i18n("Cancel"), self)
		hl.addWidget(self.cancelButton)

		QObject.connect(self.exportButton, SIGNAL("clicked()"), self, SLOT("accept()"))
		QObject.connect(self.cancelButton, SIGNAL("clicked()"), self, SLOT("reject()"))
		QObject.connect(self.exportPathButton, SIGNAL("clicked()"), self.slotShowPathDialog)
		QObject.connect(self.sizeCombo, SIGNAL("activated(int)"), self.slotChangeSize)
		QObject.connect(self.sizeRadio, SIGNAL("toggled(bool)"), self.slotRadioToggle)
		QObject.connect(self.customRadio, SIGNAL("toggled(bool)"), self.slotRadioToggle)
		QObject.connect(self.widthEdit, SIGNAL("textChanged(const QString &)"), self.slotSizeEdit)
		QObject.connect(self.heightEdit, SIGNAL("textChanged(const QString &)"), self.slotSizeEdit)
		# init dialog settings
		self.initSizeChoices()
		
		self.__serialize(True)

	def done(self, result):
		if result == QDialog.Accepted:
			if (self.sizeRadio.isChecked()):
				self.size = self.sizes[self.sizeCombo.currentItem()][1]
			else:
				self.size = QSize(int(str(self.widthEdit.text())), int(str(self.heightEdit.text())))
			self.exportDir = str(os.path.join(str(self.exportEdit.text()),self.albumPath[-1]))
			self.useTitle = self.titleRadio.isChecked()
			
			self.__serialize(False)
			
		return QDialog.done(self, result)

	def __serialize(self, bRestore):
		# export folder settings
		self.settings.serializeEdit(self.exportEdit, "AlbumExport", "folder", bRestore, os.path.expanduser('~/My Documents'))
		# size settings
		self.settings.serializeEdit(self.widthEdit, "AlbumExport", "width", bRestore)
		self.settings.serializeEdit(self.heightEdit, "AlbumExport", "height", bRestore)
		self.settings.serializeCombo(self.sizeCombo, "AlbumExport", "sizeIndex", bRestore)
		self.settings.serializeRadioGroup([self.sizeRadio, self.customRadio], "AlbumExport", "usesize", bRestore)
		# filenaming settings
		self.settings.serializeRadioGroup([self.titleRadio, self.origRadio], "AlbumExport", "usetitle", bRestore)


	def export(self):
		# get album total

		album = self.lib.getAlbum(self.albumPath)
		keys = album['KeyList']
		total = len(keys)

		progress = QProgressDialog(i18n("Exporting Album"),None,total,None,"progress",True)
		progress.setMinimumDuration(0)
		progress.setCaption(i18n("Creating Photo List"))
		progress.setTotalSteps(total)
		progress.setCancelButtonText(i18n("Cancel"))
		progress.setProgress(0)
		
		dir = self.exportEdit.text()
		LUtils.ensureDirectory(str(self.exportDir))
		copies = [] # array files we have saved so far
		
		progress.setCaption(i18n("Saving Files"))
		for key in keys:
			dict = self.lib.getImage(key)
			# ensure no duplicate filenames
			if self.useTitle:
				title = dict['Caption']
			else:
				(p, title) = os.path.split(dict['ImagePath'])
			progress.setLabelText("%s %s" % (i18n("Exporting "), title))
			# check for duplicate already in copies
			filename = os.path.join(self.exportDir, title)
			filename = self.fixName(copies, filename)
			if self.size:
				LUtils.downsizeImage(dict['ImagePath'], filename, self.size)
			else:
				LUtils.safeCopy(dict['ImagePath'], filename)
			copies.append(filename)
			progress.setProgress(progress.progress()+1)
			if progress.wasCancelled():
				return


class LSelectionExport(LExportBase):
	def __init__(self, parent, lib, keys):
		LExportBase.__init__(self, parent, keys)
		self.setCaption(i18n("Export Selection"))
		
		gl = QGridLayout(self,14,2,5)

		r = self.buildPathInGrid(gl, 0, 0)
		r = self.buildSpacer(gl, r+1, 0, 10)
		r = self.buildNameInGrid(gl, r+1, 0)
		r = self.buildSpacer(gl, r+1, 0, 10)
		r = self.buildSizeInGrid(gl, r+1, 0)
		r = self.buildSpacer(gl, r+1, 0, 10)
		r = self.buildInfoLabelsInGrid(gl, r+1, 0)	

		r = self.buildSpacer(gl, r+1, 0, 10)
		
		hl = QHBoxLayout()
		gl.addLayout(hl,r+1,1)
		hl.addStretch()
		self.exportButton = QPushButton(i18n("Export"), self)
		hl.addWidget(self.exportButton)
		self.exportButton.setDefault(True)
		hl.addSpacing(5)
		self.cancelButton = QPushButton(i18n("Cancel"), self)
		hl.addWidget(self.cancelButton)

		QObject.connect(self.exportButton, SIGNAL("clicked()"), self, SLOT("accept()"))
		QObject.connect(self.cancelButton, SIGNAL("clicked()"), self, SLOT("reject()"))
		QObject.connect(self.exportPathButton, SIGNAL("clicked()"), self.slotShowPathDialog)
		QObject.connect(self.sizeCombo, SIGNAL("activated(int)"), self.slotChangeSize)
		QObject.connect(self.sizeRadio, SIGNAL("toggled(bool)"), self.slotRadioToggle)
		QObject.connect(self.customRadio, SIGNAL("toggled(bool)"), self.slotRadioToggle)
		QObject.connect(self.widthEdit, SIGNAL("textChanged(const QString &)"), self.slotSizeEdit)
		QObject.connect(self.heightEdit, SIGNAL("textChanged(const QString &)"), self.slotSizeEdit)
		# init dialog settings
		self.initSizeChoices()
		
		self.__serialize(True);
	
		
	def done(self, result):
		if result == QDialog.Accepted:
			if (self.sizeRadio.isChecked()):
				self.size = self.sizes[self.sizeCombo.currentItem()][1]
			else:
				self.size = QSize(int(str(self.widthEdit.text())), int(str(self.heightEdit.text())))
			self.exportDir = str(self.exportEdit.text())
			self.useTitle = self.titleRadio.isChecked()
			
			self.__serialize(False)
			
			
		return QDialog.done(self, result)

	def __serialize(self, bRestore):
		# export folder settings
		self.settings.serializeEdit(self.exportEdit, "SelectionExport", "folder", bRestore, os.path.expanduser('~/My Documents'),)
		# size settings
		self.settings.serializeEdit(self.widthEdit, "SelectionExport", "width", bRestore)
		self.settings.serializeEdit(self.heightEdit, "SelectionExport", "height", bRestore)
		self.settings.serializeCombo(self.sizeCombo, "SelectionExport", "sizeIndex", bRestore)
		self.settings.serializeRadioGroup([self.sizeRadio, self.customRadio], "SelectionExport", "usesize", bRestore)
		# filenaming settings
		self.settings.serializeRadioGroup([self.titleRadio, self.origRadio], "AlbumExport", "usetitle", bRestore)

	def export(self):
		# get album total

		total = len(self.keys)

		progress = QProgressDialog(i18n("Exporting Selection"),None,total,None,"progress",True)
		progress.setMinimumDuration(0)
		progress.setCaption(i18n("Creating Photo List"))
		progress.setTotalSteps(total)
		progress.setCancelButtonText(i18n("Cancel"))
		progress.setProgress(0)
		
		dir = self.exportEdit.text()
		LUtils.ensureDirectory(str(self.exportDir))
		copies = [] # array files we have saved so far
		
		progress.setCaption(i18n("Saving Files"))
		for key in self.keys:
			dict = self.lib.getImage(key)
			# ensure no duplicate filenames
			if self.useTitle:
				title = dict['Caption']
			else:
				(p, title) = os.path.split(dict['ImagePath'])
			progress.setLabelText("%s %s" % (i18n("Exporting "), title))
			# check for duplicate already in copies
			filename = os.path.join(self.exportDir, title)
			filename = self.fixName(copies, filename)
			if self.size:
				LUtils.downsizeImage(dict['ImagePath'], filename, self.size)
			else:
				LUtils.safeCopy(dict['ImagePath'], filename)
			copies.append(filename)
			progress.setProgress(progress.progress()+1)
			if progress.wasCancelled():
				return


class LEmailExport(LExportBase):
	def __init__(self, parent, keys):
		LExportBase.__init__(self, parent, keys)
		self.setCaption(i18n("Send Email"))

		r = 0
		gl = QGridLayout(self,9,2,5)

		r = self.buildSizeInGrid(gl, r+1, 0)
		r = self.buildSpacer(gl, r+1, 0, 10)
		r = self.buildInfoLabelsInGrid(gl, r+1, 0)
		r = self.buildSpacer(gl, r+1, 0, 10)
		
		hl = QHBoxLayout()
		gl.addLayout(hl,r+1,1)
		hl.addStretch()
		self.exportButton = QPushButton(i18n("Send Email..."), self)
		hl.addWidget(self.exportButton)
		self.exportButton.setDefault(True)
		hl.addSpacing(5)
		self.cancelButton = QPushButton(i18n("Cancel"), self)
		hl.addWidget(self.cancelButton)
		
		QObject.connect(self.exportButton, SIGNAL("clicked()"), self, SLOT("accept()"))
		QObject.connect(self.cancelButton, SIGNAL("clicked()"), self, SLOT("reject()"))
		QObject.connect(self.sizeCombo, SIGNAL("activated(int)"), self.slotChangeSize)
		QObject.connect(self.sizeRadio, SIGNAL("toggled(bool)"), self.slotRadioToggle)
		QObject.connect(self.customRadio, SIGNAL("toggled(bool)"), self.slotRadioToggle)
		QObject.connect(self.widthEdit, SIGNAL("textChanged(const QString &)"), self.slotSizeEdit)
		QObject.connect(self.heightEdit, SIGNAL("textChanged(const QString &)"), self.slotSizeEdit)

		# init dialog settings
		self.initSizeChoices()
		self.__serialize(True)
		
	def done(self, result):
		if result == QDialog.Accepted:
			self.__serialize(False)
		return QDialog.done(self, result)
		
	def __serialize(self, bRestore):
		# size settings
		self.settings.serializeEdit(self.widthEdit, "EmailExport", "width", bRestore)
		self.settings.serializeEdit(self.heightEdit, "EmailExport", "height", bRestore)
		self.settings.serializeCombo(self.sizeCombo, "EmailExport", "sizeIndex", bRestore)
		self.settings.serializeRadioGroup([self.sizeRadio, self.customRadio], "EmailExport", "usesize", bRestore)
		
		
	def export(self):
		import urllib
		progress = QProgressDialog(i18n("Create EMail"),None,len(self.keys),None,"progress",True)
		progress.setMinimumDuration(0)
		progress.setCaption(i18n("Creating Photo List"))
		progress.setTotalSteps(len(self.keys))
		progress.setProgress(0)
		files = []

		for key in self.keys:
			dict = self.lib.getImage(key)
			progress.setLabelText(i18n("Generating %s" % (dict['Caption'])))
			if self.size:
				(path, filename) = os.path.split(dict['ImagePath'])
				tmpEnv = os.environ.get('TMP','/tmp')
				tmpPath = os.path.join(tmpEnv, filename)
				LUtils.downsizeImage(dict['ImagePath'], tmpPath, self.size)
				#qsl.append(str(tmpPath))
				files.append("file://"+urllib.quote(str(tmpPath)))
			else:
				#qsl.append(str(dict['ImagePath']))
				files.append("file://"+urllib.quote(str(dict['ImagePath'])))
			progress.setProgress(progress.progress()+1)
			if progress.wasCancelled():
				return
# when invokeMailer supports mozilla, we can add this back in
##~ 		app.invokeMailer("","","","", "", "", qsl)
		comma = ","
		strFiles = comma.join(files)
		# is mozilla running?
		try:
			mozillaLines = []
			mozillaLines = backtick("pidof mozilla-bin")
		except:
			print "ERROR"
			pass
			
		print mozillaLines
			
		if len(mozillaLines):
			print "mozilla -remote \"xfeDoCommand(composeMessage, attachment='%s')\" &" % (strFiles)
			os.system("mozilla -remote \"xfeDoCommand(composeMessage, attachment='%s')\" &" % (strFiles))
		else:
			print "mozilla -compose attachment=\"'%s'\" &" %(strFiles)
			os.system("mozilla -compose attachment=\"'%s'\" &" %(strFiles))

class LHtmlExport(LExportBase):
	def __init__(self, parent, albumPath, keys):
		LExportBase.__init__(self, parent, keys)
		self.setCaption(i18n("Create Web Pages"))
		self.albumName = albumPath[-1]
		self.albumPath = albumPath
		
		gl = QGridLayout(self,13,2,5)
		r = 0
		r = self.buildPathInGrid(gl, r, 0)
		r = self.buildSpacer(gl, r+1, 0, 10)
		r = self.buildTemplateInGrid(gl, r+1, 0)
		r = self.buildSpacer(gl, r+1, 0, 10)
		r = self.buildSizeInGrid(gl, r+1, 0)
		r = self.buildSpacer(gl, r+1, 0, 10)
		r = self.buildInfoLabelsInGrid(gl, r+1, 0)	

		r = self.buildSpacer(gl, r+1, 0, 10)
		
		hl = QHBoxLayout()
		gl.addLayout(hl,r+1,1)
		hl.addStretch()
		self.exportButton = QPushButton(i18n("Create"), self)
		hl.addWidget(self.exportButton)
		self.exportButton.setDefault(True)
		hl.addSpacing(5)
		self.cancelButton = QPushButton(i18n("Cancel"), self)
		hl.addWidget(self.cancelButton)

		QObject.connect(self.exportButton, SIGNAL("clicked()"), self, SLOT("accept()"))
		QObject.connect(self.cancelButton, SIGNAL("clicked()"), self, SLOT("reject()"))
		QObject.connect(self.exportPathButton, SIGNAL("clicked()"), self.slotShowPathDialog)
		QObject.connect(self.sizeCombo, SIGNAL("activated(int)"), self.slotChangeSize)
		QObject.connect(self.sizeRadio, SIGNAL("toggled(bool)"), self.slotRadioToggle)
		QObject.connect(self.customRadio, SIGNAL("toggled(bool)"), self.slotRadioToggle)
		QObject.connect(self.widthEdit, SIGNAL("textChanged(const QString &)"), self.slotSizeEdit)
		QObject.connect(self.heightEdit, SIGNAL("textChanged(const QString &)"), self.slotSizeEdit)

##~ 		QObject.connect(self.templateButton, SIGNAL("clicked()"), self.slotShowTemplatePathDialog)

		templates = self.__findAvailableTemplates()
		for t in templates:	self.templateCombo.insertItem(t)

		# init dialog settings
		self.initSizeChoices()
		self.__findAvailableTemplates()
		
		self.__serialize(True)
		
		
	def buildTemplateInGrid(self, grid, r, c):
		l = QLabel("Template:", self)
		grid.addWidget(l,r,c)
		hl = QHBoxLayout()
		grid.addLayout(hl,r,c+1)
		self.templateCombo = QComboBox(self)
		hl.addWidget(self.templateCombo)
##~ 		self.templateButton = QPushButton("...", self)
##~ 		hl.addWidget(self.templateButton)
		return r
		
	def slotShowTemplatePathDialog(self):
		path = KFileDialog.getExistingDirectory(self.exportEdit.text(), None, i18n("Select an album template"))
		if path:
			self.templateEdit.setText(path)

	
	def done(self, result):
		if result == QDialog.Accepted:
			if (self.sizeRadio.isChecked()):
				self.size = self.sizes[self.sizeCombo.currentItem()][1]
			else:
				self.size = QSize(int(str(self.widthEdit.text())), int(str(self.heightEdit.text())))
			self.__serialize(False)
			# save export path /my/path/albumname -> /my/path
			
			# save template path
		return QDialog.done(self, result)
		
	def __serialize(self, bRestore):
		self.__serializeNonAlbumSettings(bRestore)
		self.__serializeAlbumSettings(bRestore)
	
	def __serializeAlbumSettings(self,bRestore):
		# export folder settings
		self.lib.serializeAlbumEdit(self.albumPath, self.exportEdit, "HtmlExport", "folder", bRestore, str(os.path.join(self.lib.libraryPath(),"web pages", self.albumPath[-1])))
		# size settings
		self.lib.serializeAlbumEdit(self.albumPath, self.widthEdit, "HtmlExport", "width", bRestore)
		self.lib.serializeAlbumEdit(self.albumPath, self.heightEdit, "HtmlExport", "height", bRestore)
		self.lib.serializeAlbumCombo(self.albumPath, self.sizeCombo, "HtmlExport", "sizeIndex", bRestore)
		self.lib.serializeAlbumRadioGroup(self.albumPath, [self.sizeRadio, self.customRadio], "HtmlExport", "usesize", bRestore)
		# template settings
		self.lib.serializeAlbumCombo(self.albumPath, self.templateCombo, "HtmlExport", "template", bRestore, "standard")
			
	def __serializeNonAlbumSettings(self, bRestore):
		# export folder settings
##~ 		self.settings.serializeEdit(self.exportEdit, "HtmlExport", "folder", bRestore, str(os.path.join(self.lib.libraryPath(),"web pages", self.albumPath[-1])))
		# size settings
		self.settings.serializeEdit(self.widthEdit, "HtmlExport", "width", bRestore)
		self.settings.serializeEdit(self.heightEdit, "HtmlExport", "height", bRestore)
		self.settings.serializeCombo(self.sizeCombo, "HtmlExport", "sizeIndex", bRestore)
		self.settings.serializeRadioGroup([self.sizeRadio, self.customRadio], "HtmlExport", "usesize", bRestore)
		# template settings
##~ 		self.settings.serializeCombo(self.templateCombo, "HtmlExport", "template", bRestore, "standard")

	def __xslXform(self, xmlString, templatePath, outputPath):
##~ 		libxsltmod.translate_to_file('f', templatePath, 's', xmlString, outputPath, None)
		tmpEnv = os.environ.get('TMP','/tmp')
		tmpPath = os.path.join(tmpEnv, "lphototmp.xml")
		ouf = open(tmpPath, mode="w")
		ouf.write(xmlString)
		ouf.close()
		os.system("xsltproc -o \"%s\" \"%s\" \"%s\"" % (outputPath, templatePath, tmpPath))

	def export(self):
		# create directory
		tmpltPath = str(self.templateCombo.currentText())
		exportDir = str(self.exportEdit.text())
		thumbDir = os.path.join(exportDir, "thumbs")
		LUtils.ensureDirectory(thumbDir)
		imageDir = os.path.join(exportDir, "images")
		LUtils.ensureDirectory(imageDir)
		
		indexHTML = os.path.join(exportDir, "index.html")
		
		progress = QProgressDialog(i18n("Creating Web Pages"),None,len(self.keys),None,"progress",True)
		progress.setMinimumDuration(0)
		progress.setCaption(i18n("Creating Photo List"))
		progress.setTotalSteps(len(self.keys)+1)
		progress.setCancelButtonText(i18n("Cancel"))
		progress.setProgress(0)
		
		photos = map(self.lib.getImage, self.keys)
				
		progress.setCaption(i18n("Exporting Files"))
		for dict in photos:
			progress.setLabelText(i18n("Exporting %s"%(dict['Caption'])))
			(path, filename) = os.path.split(dict['ImagePath'])

			# generate images
			thumbPath = os.path.join(thumbDir, filename)
			imagePath = os.path.join(imageDir, filename)
			LUtils.downsizeImage(dict['ImagePath'], thumbPath, QSize(180,180))
			
			if self.size:
				LUtils.downsizeImage(dict['ImagePath'], imagePath, self.size)
			else:
				shutil.copy2(dict['ImagePath'], imagePath)
			progress.setProgress(progress.progress()+1)
			if progress.wasCancelled():
				return

		progress.setCaption(i18n("Generating Template"))
		progress.setLabelText(i18n("Generating Template"))
		template = __import__("templates/%s/template"%(tmpltPath))
		template.construct(paths.template(tmpltPath), exportDir, photos, self.albumPath[-1])
		progress.setProgress(progress.progress()+1)
			
		# launch mozilla to this page
		os.system("nslauncher \"file://%s\" &" % (indexHTML))

	def __findAvailableTemplates(self):
		import glob, re, os
		files = glob.glob(paths.templatePath()+"/*/template.py")
		match = re.compile(r".*/(.*)/template\.py$", re.IGNORECASE)          
		l = []
		
		for f in files:
			mc = match.search(f)
			if mc:
				l.append(mc.group(1))
		return l
